function cof = sgfsp(N,p,q,T,s)
%Function:
%       Savitzky-Golay Smoothing and Differentiation Filter with a given Sampling Period 
%Usage:
%       N    = Number of data                                               > = 2  integer
%       p    = polynomial order                                             > = 1  integer 
%       q    =  differentation order (0=smoothing,1=first derivative,...),  > = 0  integer
%       T    = sampling period                                              > 0    float
%       s    = estimation position, float, s =  0 : the last data; s = N*T: the fisrt data; s = N*T/2: the middle data                
%      cof   = filter coefficients
%Author:
%       Quan Quan <qq_buaa@buaa.edu.cn> 2015-3-1
%       Department of Automatic Control
%       Beihang University, Beijing 100191, P. R. China  
%Reference:
%[1] Quan Quan, Kai-Yuan Cai, "Time-domain analysis of the SavitzkyCGolay filters," Digital Signal Processing 22 (2012) 238C245.

if N<2 || N~=fix(N)
 error('Number of data is too small,or is not an integer !');      
end

if p<1 || p~=fix(p)
 error('Polynomial order is too small, or is not an integer !');      
end

if q<0 || q~=fix(q)
 error('Differentation order is negative,or is not an integer !');      
end

if T <= 0
 error('Sampling period is zero or negative !');      
end

N = N - 1;

Phi = zeros(N+1,p+1);

for i = 0 : N
  for j = 0 : p 
      Phi(i+1,j+1) = (i*T)^j;
    end
end

L = zeros(p+1,1);
for i = 0 : p
    if i-q<0
       L(i+1) = 0;
    else
       L(i+1) = prod(i:-1:i+1-q)*s^(i-q);
    end
end

% See Equ.(8) in [1]
cof = Phi*inv(Phi'*Phi)*L;